# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-04-22 15:13:52
@LastEditors: ChenXiaolei
@Description: uuid helper
"""
import uuid


class UUIDHelper:
    @classmethod
    def get_uuid(self):
        """
        @description: 获取uuid4
        @return: uuid字符串
        @last_editors: ChenXiaolei
        """
        return str(uuid.uuid4())


# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-25 21:58:58
@LastEditTime: 2020-04-26 11:14:31
@LastEditors: ChenXiaolei
@Description: 
"""

from . import base_api_handler
from . import base_handler


# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-04-16 21:05:11
@LastEditors: ChenXiaolei
@Description: redis helper
"""

import redis


class RedisHelper:
    @classmethod
    def redis_init(self, host, port, db, password=None):
        """
        @description: 从redis连接池中创建对象
        @param host:主机地址
        @param port:端口
        @param db:redis_db
        @param password:授权密码
        @return: redis客户端对象
        @last_editors: ChenXiaolei
        """
        pool = redis.ConnectionPool(host=host,
                                    port=port,
                                    db=db,
                                    password=password)
        redis_client = redis.Redis(connection_pool=pool, decode_responses=True)
        return redis_client



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-04-16 16:49:56
@LastEditors: ChenXiaolei
@Description: 
"""
from elasticsearch import Elasticsearch


class ESHelper:
    """
    Elasticsearch==7.x 二次封装操作类 
    """
    def __init__(self,
                 host,
                 port,
                 index=None,
                 auth_username=None,
                 auth_password=None):
        self._es_client = Elasticsearch(host,
                                        http_auth=(auth_username,
                                                   auth_password),
                                        port=port)
        """
        @description: Elasticsearch 初始化
        @param host:集群host
        @param port:端口
        @param index:ES索引
        @param auth_username:授权用户名
        @param auth_password:授权密码
        @return: ESHelper类
        @last_editors: ChenXiaolei
        """
        if index:
            self.index = index

    def _get_index(self, index):
        """
        @description: 判断传参index是否为空，如果为空则从初始化属性获取
        @param index: es_index
        @return: es_index
        @last_editors: ChenXiaolei
        """
        if not index:
            if not self.index:
                raise Exception("index is not configured")
            return self.index
        return index

    def get_es_client(self):
        """
        @description: 获得es原始客户端对象
        @param 无
        @return: es客户端对象
        @last_editors: ChenXiaolei
        """
        return self._es_client

    def insert(self, body, index=None, id=None):
        """
        @description: 写入数据
        @param body: doc 字典数据类型
        @return: 执行返回结果
        @last_editors: ChenXiaolei
        """
        return self._es_client.index(index=self._get_index(index),
                                     body=body,
                                     id=id)

    def bulk(self, body, index=None):
        """
        @description: 批量操作
            demo:
            body = [{"index":{"_index":"test_index"}}]
            doc = {}
            doc["field"]="value"
            body.append(doc)
            es_client.bulk(body=body)

        @param body: 执行body
        @return: 执行返回结果
        @last_editors: ChenXiaolei
        """
        return self._es_client.bulk(index=self._get_index(index), body=body)

    def delete_by_id(self, id, index=None):
        """
        @description: 根据es_id删除数据
        @param id: es_id
        @param index: es_index 
        @return: 执行返回结果
        @last_editors: ChenXiaolei
        """
        return self._es_client.delete(index=self._get_index(index), id=id)

    def delete_by_query(self, body, index=None):
        """
        @description: 根据dsl条件删除数据
        @param body: 执行dsl
        @return: 执行返回结果
        @last_editors: ChenXiaolei
        """
        return self._es_client.delete_by_query(index=self._get_index(index),
                                               body=body)

    def update(self, id, body, index=None):
        """
        @description: 根据_id更新数据
        @param id: es_id
        @param body: 执行dsl
        @param index: es_index 
        @return: 执行返回结果
        @last_editors: ChenXiaolei
        """
        return self._es_client.update(index=self._get_index(index),
                                      id=id,
                                      body=body)

    def update_by_query(self, body, index=None):
        """
        @description: 根据dsl条件更新数据
        @param body: 执行dsl
        @param index: es_index 
        @return: 执行返回结果
        @last_editors: ChenXiaolei
        """
        return self._es_client.update_by_query(index=self._get_index(index),
                                               body=body)

    def search(self, body, index=None):
        """
        @description: 根据dsl查询数据
        @param body: 执行dsl
        @param index: es_index 
        @return: 查询返回结果
        @last_editors: ChenXiaolei
        """
        return self._es_client.search(index=self._get_index(index), body=body)

    def exists_source(self, id, index=None):
        """
        @description: 根据_id获取数据是否已存在
        @param id: es_id
        @param index: es_index 
        @return: 返回 True or False
        @last_editors: ChenXiaolei
        """
        return self._es_client.exists_source(index=self._get_index(index),
                                             id=id)



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-04-25 22:53:42
@LastEditors: ChenXiaolei
@Description: 
"""
import base64
import binascii
from Crypto.Cipher import AES


class CryptoHelper:
    """
    @description: 加密帮助类
    """
    @classmethod
    def md5_encrypt(self, source, salt=""):
        """
        @description: md5加密，支持加盐算法
        @param source: 需加密的字符串
        @param salt: 加盐算法参数值
        @return: md5加密后的字符串
        @last_editors: ChenXiaolei
        """
        if not source.strip():
            return ""
        import hashlib
        encrypt = hashlib.md5()
        encrypt.update(
            (source + salt).encode('utf-8')
        )  # 参数必须是byte类型，否则报Unicode-objects must be encoded before hashing错误
        md5value = encrypt.hexdigest()
        return md5value

    @classmethod
    def md5_encrypt_int(self, source, salt=""):
        """
        @description: md5加密，返回数值
        @param source: 需加密的字符串
        @param salt: 加盐算法参数值
        @return: md5加密后的数值
        @last_editors: ChenXiaolei
        """
        md5_16 = self._convert_md5(self.md5_encrypt(source, salt))
        hash_code_start = int.from_bytes(md5_16[0:8],
                                         byteorder='little',
                                         signed=True)
        hash_code_end = int.from_bytes(md5_16[8:16],
                                       byteorder='little',
                                       signed=True)
        return hash_code_start ^ hash_code_end

    @classmethod
    def _convert_md5(self, origin):
        """
        @description: md5字符串转16进制数组
        @param origin: 原md5字符串
        @return: 16进制数组
        @last_editors: ChenXiaolei
        """
        result = []
        s = ""
        for i in range(len(origin)):
            s += origin[i]
            if i % 2 != 0:
                int_hex = int(s, 16)
                result.append(int_hex)
                s = ""

        return result

    @classmethod
    def base64_encode(self, source):
        """
        @description: base64加密
        @param source: 需加密的字符串
        @return: 加密后的字符串
        @last_editors: ChenXiaolei
        """
        if not source.strip():
            return ""
        import base64
        encode_string = base64.b64encode(source.encode(encoding='utf-8'))
        return encode_string

    @classmethod
    def base64_decode(self, source):
        """
        @description: base64解密
        @param source: 需加密的字符串
        @return: 解密后的字符串
        @last_editors: ChenXiaolei
        """
        if not source.strip():
            return ""
        import base64
        decode_string = base64.b64decode(source)
        return decode_string

    @classmethod
    def aes_encrypt(self, data, password):
        """
        @description: AES加密,ECB & PKCS7
        @param {type} 
        @return: 
        @last_editors: ChenXiaolei
        """
        if isinstance(password, str):
            password = password.encode('utf8')

        bs = AES.block_size
        pad = lambda s: s + (bs - len(s) % bs) * chr(bs - len(s) % bs)
        cipher = AES.new(password, AES.MODE_ECB)
        data = cipher.encrypt(pad(data).encode('utf8'))
        encrypt_data = binascii.b2a_hex(data)  # 输出hex
        encrypt_data = base64.b64encode(data)  # 输出Base64格式
        return encrypt_data.decode('utf8')

    @classmethod
    def aes_decrypt(self, decrData, password):
        """
        @description: 
        @param {type} 
        @return: 
        @last_editors: ChenXiaolei
        """
        if isinstance(password, str):
            password = password.encode('utf8')

        cipher = AES.new(password, AES.MODE_ECB)
        plain_text = cipher.decrypt(binascii.a2b_hex(decrData))
        return plain_text.decode('utf8').rstrip('\0')



'''
@Author: ChenXiaolei
@Date: 2020-03-06 23:17:54
@LastEditTime: 2020-03-21 16:50:31
@LastEditors: ChenXiaolei
@Description: time helper
@FilePath: /python_base_framework/libs./seven_framework/time.py
'''
# -*- coding: utf-8 -*-

import time
import datetime
from dateutil.relativedelta import relativedelta


class TimeHelper:
    @classmethod
    def format_time_to_datetime(self,
                                format_time=None,
                                format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 时间字符串转datetime
        @param format_time: 格式化时间，如果未传则取服务器当前时间
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        if not format_time:
            return datetime.datetime.now()
        return datetime.datetime.strptime(format_time, format)

    @classmethod
    def datetime_to_format_time(self, dt, format='%Y-%m-%d %H:%M:%S'):
        """
        @description: datetime转时间字符串
        @param dt: datetime格式时间
        @param format: 格式化时间格式
        @return: 时间字符串
        @last_editors: ChenXiaolei
        """
        return dt.strftime(format)

    @classmethod
    def datetime_to_timestamp(self,
                              dt,
                              format='%Y-%m-%d %H:%M:%S',
                              out_ms=False):
        """
        @description: datetime转unix时间戳
        @param dt: datetime格式时间
        @param format: 格式化时间格式
        @return: unix时间戳
        @last_editors: ChenXiaolei
        """
        if out_ms:
            return int(
                time.mktime(dt.timetuple()) * 1000.0 + dt.microsecond / 1000.0)
        return int(time.mktime(dt.timetuple()))

    @classmethod
    def format_time_to_timestamp(self,
                                 format_time=None,
                                 format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 格式化时间转为Unix时间戳
        @param format_time: 格式化时间，如果未传则返回服务器当前时间
        @param format: 格式化时间格式
        @return: Unix时间戳
        @last_editors: ChenXiaolei
        """
        if format_time:
            time_tuple = time.strptime(format_time, format)
            result = time.mktime(time_tuple)
            return int(result)
        return int(time.time())

    @classmethod
    def timestamp_to_format_time(self,
                                 timestamp=None,
                                 format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 时间戳转格式化时间
        @param timestamp: unix时间戳 
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        if timestamp:
            time_tuple = time.localtime(timestamp)
            result = time.strftime(format, time_tuple)
            return result
        else:
            return time.strftime(format)

    @classmethod
    def timestamp_to_datetime(self, timestamp=time.time()):
        """
        @description: unix时间戳转datetime
        @param timestamp: unix时间戳，如果没传默认取服务器当前时间
        @return: datetime
        @last_editors: ChenXiaolei
        """
        if self.is_ms_timestamp(timestamp):
            timestamp = timestamp / 1000
        return datetime.datetime.fromtimestamp(timestamp)

    @classmethod
    def get_now_timestamp(self):
        """
        @description: 获取当前时间戳
        @return: 时间戳
        @last_editors: ChenXiaolei
        """
        return int(time.time())

    # 获取当前时间格式
    @classmethod
    def get_now_datetime(self):
        """
        @description: 获取当前时间格式
        @return: 格式化时间
        @last_editors: ChenXiaolei
        """
        return datetime.datetime.now()

    @classmethod
    def get_now_format_time(self, format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 获取当前时间格式
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        return datetime.datetime.now().strftime(format)

    @classmethod
    def is_ms_timestamp(self, timestamp):
        """
        @description: 时间戳是否为毫秒时间戳
        @param timestamp: unix时间戳 
        @return: bool
        @last_editors: ChenXiaolei
        """
        if timestamp and len(str(int(timestamp))) > 10:
            return True
        else:
            return False

    @classmethod
    def add_seconds_by_timestamp(self, timestamp=None, second=1):
        """
        @description: 为时间戳增加秒数
        @param timestamp: unix时间戳 
        @param second: 秒数
        @return: unix时间戳
        @last_editors: ChenXiaolei
        """
        if not timestamp:
            timestamp = self.get_now_timestamp()

        # 毫秒时间戳
        if self.is_ms_timestamp(timestamp):
            return timestamp + (second * 1000)
        else:
            return timestamp + second

    @classmethod
    def add_minutes_by_timestamp(self, timestamp=None, minute=1):
        """
        @description: 为时间戳增加分钟数
        @param timestamp: unix时间戳 
        @param minute: 分钟数
        @return: unix时间戳
        @last_editors: ChenXiaolei
        """
        return self.add_seconds_by_timestamp(timestamp, minute * 60)

    @classmethod
    def add_hours_by_timestamp(self, timestamp=None, hour=1):
        """
        @description: 为时间戳增加小时数
        @param timestamp: unix时间戳 
        @param hour: 小时数
        @return: unix时间戳
        @last_editors: ChenXiaolei
        """
        return self.add_seconds_by_timestamp(timestamp, hour * 3600)

    @classmethod
    def add_days_by_timestamp(self, timestamp=None, day=1):
        """
        @description: 为时间戳增加天数
        @param timestamp: unix时间戳 
        @param day: 天数
        @return: unix时间戳
        @last_editors: ChenXiaolei
        """
        return self.add_seconds_by_timestamp(timestamp, day * 86400)

    @classmethod
    def add_months_by_timestamp(self, timestamp=None, months=1):
        """
        @description: 为时间戳增加月数
        @param timestamp: unix时间戳 
        @param months: 月数
        @return: unix时间戳
        @last_editors: ChenXiaolei
        """
        if not timestamp:
            timestamp = time.time()

        dt = self.timestamp_to_datetime(timestamp)
        return self.datetime_to_timestamp(
            dt + relativedelta(months=months),
            out_ms=self.is_ms_timestamp(timestamp))

    @classmethod
    def add_years_by_timestamp(self, timestamp=None, years=1):
        """
        @description: 为时间戳增加年数
        @param timestamp: unix时间戳 
        @param years: 年数
        @return: unix时间戳
        @last_editors: ChenXiaolei
        """
        if not timestamp:
            timestamp = time.time()

        dt = self.timestamp_to_datetime(timestamp)
        return self.datetime_to_timestamp(
            dt + relativedelta(years=years),
            out_ms=self.is_ms_timestamp(timestamp))

    @classmethod
    def add_second_by_format_time(self,
                                  format_time=None,
                                  second=1,
                                  format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 为时间格式增加秒数
        @param format_time: 格式化时间，如果未传则取服务器当前时间
        @param second: 秒数 
        @return: datetime
        @last_editors: ChenXiaolei
        """
        return (self.format_time_to_datetime(format_time, format) +
                datetime.timedelta(seconds=second)).strftime(format)

    @classmethod
    def add_minutes_by_format_time(self,
                                   format_time=None,
                                   minute=1,
                                   format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 为时间格式增加分钟数
        @param format_time: 格式化时间，如果未传则取服务器当前时间
        @param minute: 分钟数
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        return (self.format_time_to_datetime(format_time, format) +
                datetime.timedelta(minutes=minute)).strftime(format)

    @classmethod
    def add_hours_by_format_time(self,
                                 format_time=None,
                                 hour=1,
                                 format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 为时间格式增加小时数
        @param format_time: 格式化时间，如果未传则取服务器当前时间
        @param hour: 小时数
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        return (self.format_time_to_datetime(format_time, format) +
                datetime.timedelta(hours=hour)).strftime(format)

    @classmethod
    def add_days_by_format_time(self,
                                format_time=None,
                                day=1,
                                format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 为时间格式增加分钟数
        @param format_time: 格式化时间，如果未传则取服务器当前时间
        @param day: 天数
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        return (self.format_time_to_datetime(format_time, format) +
                datetime.timedelta(days=day)).strftime(format)

    @classmethod
    def add_months_by_format_time(self,
                                  format_time=None,
                                  months=1,
                                  format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 为时间格式增加月数
        @param format_time: 格式化时间，如果未传则取服务器当前时间
        @param months: 月数
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        return (self.format_time_to_datetime(format_time, format) +
                relativedelta(months=months)).strftime(format)

    @classmethod
    def add_years_by_format_time(self,
                                 format_time=None,
                                 years=1,
                                 format='%Y-%m-%d %H:%M:%S'):
        """
        @description: 为时间格式增加年数
        @param format_time: 格式化时间，如果未传则取服务器当前时间
        @param years: 年数
        @param format: 格式化时间格式
        @return: datetime
        @last_editors: ChenXiaolei
        """
        return (self.format_time_to_datetime(format_time, format) +
                relativedelta(years=years)).strftime(format)



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 21:32:43
@LastEditTime: 2020-04-21 19:15:17
@LastEditors: ChenXiaolei
@Description: 
"""

import logging
import logging.handlers
import time
import os
import json
import socket
import platform

class Logger:
    """
    指定保存日志的文件路径，日志级别，以及调用文件 将日志存入到指定的文件中
    级别优先级:NOTSET < DEBUG < INFO < WARNING < ERROR < CRITICAL
    """
    def __init__(self, log_file_name, log_level, logger, host_ip="", project_name=None):
        """
        @description: 
        @param log_file_name: 日志存储文件路径
        @param log_level: 日志等级
        @param logger: 日志标识
        @param host_ip: 服务器IP
        @param project_name: 项目标志
        @last_editors: ChenXiaolei
        """
        if not project_name:
            if platform.system() == "Windows":
                project_name = os.getcwd().split('\\')[-1]
            else:
                project_name = os.getcwd().split('/')[-1]

        # 判断文件夹是否存在，不存在则创建
        path_list = log_file_name.split("/")
        path_log = log_file_name[0:log_file_name.find(path_list[len(path_list)
                                                                - 1])]
        if not os.path.isdir(path_log):
            os.mkdir(path_log)

        logging_level = ''

        if log_level.upper() == 'NOTSET':
            logging_level = logging.NOTSET
        elif log_level.upper() == 'DEBUG':
            logging_level = logging.DEBUG
        elif log_level.upper() == 'INFO':
            logging_level = logging.INFO
        elif log_level.upper() == 'WARNING':
            logging_level = logging.WARNING
        elif log_level.upper() == 'ERROR':
            logging_level = logging.ERROR
        elif log_level.upper() == 'CRITICAL':
            logging_level = logging.CRITICAL

        # 创建一个logger
        self.logger = logging.getLogger(logger)
        self.logger.setLevel(logging_level)

        # 创建handler，用于写入日志文件
        self.handler_file = logging.handlers.TimedRotatingFileHandler(
            log_file_name, 'D', 1, 10)
        self.handler_file.suffix = "%Y%m%d.log"  # 设置 切分后日志文件名的时间格式 默认 log_file_name+"." + suffix 如果需要更改需要改logging 源码
        self.handler_file.setLevel(logging_level)

        formatter = logging.Formatter(
            json.dumps({
                "record_time": "%(asctime)s",
                "level": "%(levelname)s",
                "log_msg": "%(message)s",
                "host_ip": host_ip,
                "project_name": project_name
            }))
        self.handler_file.setFormatter(formatter)

        # 创建handler，用于输出至控制台
        # 定义控制台输出handler的输出格式
        formatter = logging.Formatter(
            '[%(asctime)s][%(name)s][%(levelname)s]%(message)s')
        self.handler_console = logging.StreamHandler() # 输出到控制台的handler
        self.handler_console.setFormatter(formatter)
        
        # 给logger添加handler
        if not self.logger.handlers:
            self.logger.addHandler(self.handler_file)
            self.logger.addHandler(self.handler_console)

    def close(self):
        self.logger.removeHandler(self.handler_file)
        self.handler_file.close()

    def get_logger(self):
        return self.logger

    @classmethod
    def get_logger_by_name(self, loger_name):
        """
        @description: 通过日志标识获取logger
        @param loger_name: 日志标识
        @return: logger
        @last_editors: ChenXiaolei
        """
        return logging.getLogger(loger_name)



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-04-26 11:14:15
@LastEditors: ChenXiaolei
@Description: 
"""

from seven_framework import config
from seven_framework.crypto import *
from seven_framework.log import *
from seven_framework.mysql import *
from seven_framework.time import *
from seven_framework.uuid import *
from seven_framework.http import * 
from seven_framework.json import *

from .tornado import handler


'''
@Author: ChenXiaolei
@Date: 2020-03-14 16:12:44
@LastEditTime: 2020-03-19 15:37:32
@LastEditors: ChenXiaolei
@Description: mysql helper
@FilePath: /seven_manage_api/libs./seven_framework/mysql.py
'''
# -*- coding: utf-8 -*-
import pymysql


class MySQLHelper:
    # 对MySQLdb常用函数进行封装的类

    error_code = ''  # MySQL错误号码
    _instance = None  # 本类的实例
    conn = None  # 数据库conn
    _cur = None

    def __init__(self, dbconfig):
        """
        @description: mysql 操作类初始化
        @param dbconfig: 连接字符串
        @last_editors: ChenXiaolei
        """
        # 构造器：根据数据库连接参数，创建MySQL连接
        self._conn = pymysql.connect(
            host=dbconfig['host'],
            port=dbconfig['port'],
            user=dbconfig['user'], 
            passwd=dbconfig['passwd'],
            db=dbconfig['db'],
            charset=dbconfig['charset'])

    def query(self, sql, params=None):
        """
        @description: 执行 SELECT 语句
        @param sql: 查询语句
        @param params: 参数值
        @return: 查询结果
        @last_editors: ChenXiaolei
        """
        # 执行 SELECT 语句
        self._execute(sql, params)
        return self._cur

    def update(self, sql, params=None):
        """
        @description: 执行 UPDATE 操作
        @param sql: 查询语句
        @param params: 参数值
        @return: 执行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        self._conn.commit()
        return self._cur

    def delete(self, sql, params=None):
        """
        @description: 执行 Delete 操作
        @param sql: 查询语句
        @param params: 参数值
        @return: 执行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        self._conn.commit()
        return self._cur

    def insert(self, sql, params=None):
        """
        @description: 执行 INSERT 语句
        @param sql: 数据insert语句
        @return: 如主键为自增长int，则返回新生成的ID
        @last_editors: ChenXiaolei
        """
        # 执行 INSERT 语句。如主键为自增长int，则返回新生成的ID
        self._execute(sql, params)
        self._conn.commit()
        return self._cur._result.insert_id

    def _execute(self, sql, params=None):
        try:
            self._cur = self._conn.cursor(pymysql.cursors.DictCursor)
            if not params:
                self._cur.execute(sql)
            else:
                self._cur.execute(sql, params)

            return self._cur
        except Exception as e:
            raise Exception("ERROR:" + str(e) + " SQL：" + str(sql) + " 参数：" + str(params))

    def fetch_all_rows(self, sql, params=None):
        """
        @description: 返回结果列表
        @param sql: 查询语句
        @param params: 参数值
        @return: 结果列表
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        return self._cur.fetchall()

    def fetch_limit_rows(self, sql, params=None, current_page=1, list_rows=20):
        count = self.get_row_count(sql, params)
        pages = count / list_rows
        pages = pages + 1 if not count % list_rows == 0 else pages
        if (pages == 0): pages = 1
        if (current_page < 1): current_page = 1
        if (current_page > pages): current_page = pages
        start = (current_page - 1) * list_rows
        end = list_rows
        previous_page = current_page - 1 if current_page > 1 else 1
        next_page = current_page + 1 if current_page < int(pages) else int(pages)
        limit_sql = sql + " limit " + str(int(start)) + "," + str(int(end))
        result = {}
        result["list"] = self.fetch_all_rows(limit_sql, params)
        result["page"] = {
            "prev": previous_page,
            "next": next_page,
            "current": current_page,
            "pages": int(pages),
            "total": count,
        }

        return result

    def fetch_one_row(self, sql, params=None):
        """
        @description: 返回一行结果，然后游标指向下一行。到达最后一行以后，返回None
        @param sql: 查询语句
        @param params: 参数值
        @return: 返回一行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        return self._cur.fetchone()

    def get_row_count(self, sql, params=None):
        """
        @description: 获取结果行数
        @param sql: 查询语句
        @param params: 参数值
        @return: 返回查询行数
        @last_editors: ChenXiaolei
        """
        # 获取结果行数
        self._execute(sql, params)
        return self._cur.rowcount

    def fetch_and_commit(self, sql, params=None):
        """
        @description: 执行并commit提交
        @param sql: 查询语句
        @param params: 参数值
        @return: 返回第一行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        _row = self._cur.fetchone()
        self._conn.commit()
        return _row

    def commit(self):
        """
        @description: 数据库commit操作
        @last_editors: ChenXiaolei
        """
        # 数据库commit操作
        self._conn.commit()

    def rollback(self):
        """
        @description: 执行数据库回滚操作
        @last_editors: ChenXiaolei
        """
        # 数据库回滚操作'
        self._conn.rollback()

    def __del__(self):
        """
        @description: 执行释放资源（系统GC自动调用）
        @last_editors: ChenXiaolei
        """
        # 释放资源（系统GC自动调用）
        try:
            self._cur.close()
            self._conn.close()
        except:
            pass

    def close(self):
        """
        @description: 执行关闭数据库连接
        @last_editors: ChenXiaolei
        """
        # 关闭数据库连接
        self.__del__()



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-03-21 18:29:31
@LastEditTime: 2020-04-16 16:51:38
@LastEditors: ChenXiaolei
@Description: 
@FilePath: /python_base_framework/libs/seven_framework/config.py
"""

import os
import json

import threading
_lock = threading.Condition()


def init_config(path):
    """
    @description: 初始化配置文件
    @param path：配置文件路径，可使用物理路径或url
    @return: global app_config
    @last_editors: ChenXiaolei
    """    
    if path.lower().find("http://") > -1:
        import requests
        json_str = requests.get(path)
    else:
        with open(path, "r+", encoding="utf-8") as f:
            json_str = f.read()
    global app_config
    app_config = json.loads(json_str)


def set_value(key, value):
    """
    @description: 设置一个全局键值配置
    @param key:参数键名
    @param value:参数值
    @return: 无
    @last_editors: ChenXiaolei
    """    
    _lock.acquire()
    try:
        app_config[key] = value
    except:
        raise
    finally:
        _lock.release()


def get_value(key, default_value=None):
    """
    @description: 获得一个全局变量,不存在则返回默认值
    @param key:参数键名
    @param default_value:获取不到返回的默认值
    @return: 参数值
    @last_editors: ChenXiaolei
    """
    try:
        _lock.acquire()
        config_value = app_config[key]
    except KeyError:
        print(f"config未获取key[{key}]的配置")
        config_value = default_value
    finally:
        _lock.release()

    return config_value


# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-03-06 23:17:54
@LastEditTime: 2020-04-25 23:55:32
@LastEditors: ChenXiaolei
@Description: Handler基础类
"""

# base import
import tornado.web
import time
import datetime
import base64
import json
import io
import bleach
import asyncio

# tornado import
from tornado.concurrent import run_on_executor
from concurrent.futures import ThreadPoolExecutor

# seven_framework import
from seven_framework import *


class BaseHandler(tornado.web.RequestHandler):
    THREAD_POOL_COUNT = 500
    """
    @description: 基础handler
    @last_editors: ChenXiaolei
    """
    executor = ThreadPoolExecutor(THREAD_POOL_COUNT)

    def __init__(self, *argc, **argkw):
        """
        @description: 初始化
        @last_editors: ChenXiaolei
        """
        super(BaseHandler, self).__init__(*argc, **argkw)
        self.logger_error = Logger.get_logger_by_name("log_error")
        self.logger_info = Logger.get_logger_by_name("log_info")

    # 异步重写
    async def get(self, *args, **kwargs):
        await asyncio.get_event_loop().run_in_executor(self.executor,
                                                       self.get_async)

    async def post(self, *args, **kwargs):
        await asyncio.get_event_loop().run_in_executor(self.executor,
                                                       self.post_async)

    async def delete(self, *args, **kwargs):
        await asyncio.get_event_loop().run_in_executor(self.executor,
                                                       self.delete_async)

    async def put(self, *args, **kwargs):
        await asyncio.get_event_loop().run_in_executor(self.executor,
                                                       self.put_async)

    async def head(self, *args, **kwargs):
        await asyncio.get_event_loop().run_in_executor(self.executor,
                                                       self.head_async)

    async def options(self, *args, **kwargs):
        await asyncio.get_event_loop().run_in_executor(self.executor,
                                                       self.options_async)

    # 通用handle页面数据返回
    def render(self, template_name, **template_vars):
        """
        @description: 渲染html源码
        @param template_name: 前端模板路径
        @param **template_vars: 传递给模板的参数
        @return: 返回客户端渲染页面
        @last_editors: ChenXiaolei
        """
        html = self.render_string(template_name, **template_vars)
        self.write(html)

    def check_xsrf_cookie(self):
        """
        @description: 过滤受_xsrf影响的post请求 通过获取_xsrf|X-Xsrftoken|X-Csrftoken判断
        @last_editors: ChenXiaolei
        """
        def _time_independent_equals(a, b):
            if len(a) != len(b):
                return False
            result = 0
            if isinstance(a[0], int):  # python3 byte strings
                for x, y in zip(a, b):
                    result |= x ^ y
            else:  # python2
                for x, y in zip(a, b):
                    result |= ord(x) ^ ord(y)
            return result == 0

        if '/api/' in self.request.uri:
            return
        else:
            token = (self.get_argument("_xsrf", None)
                     or self.request.headers.get("X-Xsrftoken")
                     or self.request.headers.get("X-Csrftoken"))
            if not token:
                raise tornado.web.HTTPError(
                    403, "'_xsrf' argument missing from POST")
            _, token, _ = self._decode_xsrf_token(token)
            _, expected_token, _ = self._get_raw_xsrf_token()
            if not _time_independent_equals(
                    tornado.web.escape.utf8(token),
                    tornado.web.escape.utf8(expected_token)):
                raise tornado.web.HTTPError(
                    403, "XSRF cookie does not match POST argument")

    # 页面过滤方法，防止注入
    def html_clean(self, htmlstr):
        """
           采用bleach来清除不必要的标签，并linkify text
        """
        tags = [
            'a', 'abbr', 'acronym', 'b', 'blockquote', 'code', 'em', 'i', 'li',
            'ol', 'strong', 'ul'
        ]
        tags.extend([
            'div', 'p', 'hr', 'br', 'pre', 'code', 'span', 'h1', 'h2', 'h3',
            'h4', 'h5', 'del', 'dl', 'img', 'sub', 'sup', 'u'
            'table', 'thead', 'tr', 'th', 'td', 'tbody', 'dd', 'caption',
            'blockquote', 'section'
        ])
        attributes = {
            '*': ['class', 'id'],
            'a': ['href', 'title', 'target'],
            'img': ['src', 'style', 'width', 'height']
        }
        return bleach.linkify(
            bleach.clean(htmlstr, tags=tags, attributes=attributes))

    def write_error(self, status_code, **kwargs):
        """
        @description: 重写全局异常事件捕捉
        @last_editors: ChenXiaolei
        """
        self.logger_error.exception('error_info:')

    def get_param(self, param_name, default=""):
        """
        @description: 二次封装获取参数，经过html注入过滤
        @param param_name: 参数名
        @param default: 如果无此参数，则返回默认值
        @return: 参数值
        @last_editors: ChenXiaolei
        """
        param_ret = self.get_argument(param_name, default)
        return self.html_clean(param_ret)

    def get_remote_ip(self):
        """
        @description: 获取客户端真实IP
        @return: 客户端真实IP字符串
        @last_editors: ChenXiaolei
        """
        ip_address = ""
        if "X-Forwarded-For" in self.request.headers:
            ip_address = self.request.headers['X-Forwarded-For']
        elif "X-Real-Ip" in self.request.headers:
            ip_address = self.request.headers['X-Real-Ip']
        else:
            ip_address = self.request.remote_ip
        return ip_address

    def reponse_common(self, result, desc, data=None):
        """
        @description: 输出公共json模型
        @param result: 返回结果标识
        @param desc: 返回结果描述
        @param data: 返回结果对象，即为数组，字典
        @return: 将dumps后的数据字符串返回给客户端
        @last_editors: ChenXiaolei
        """
        template_value = {}
        template_value['result'] = result
        template_value['desc'] = desc
        template_value['data'] = data

        self.http_reponse(
            json.dumps(template_value, ensure_ascii=False, cls=JsonEncoder))

    def http_reponse(self, content):
        """
        @description: 将字符串返回给客户端
        @param content: 内容字符串
        @return: 将字符串返回给客户端
        @last_editors: ChenXiaolei
        """
        self.write(content)

    def reponse_json_success(self, data=None, desc='success'):
        """
        @description: 通用成功返回json结构
        @param data: 返回结果对象，即为数组，字典
        @param desc: 返回结果描述
        @return: 将dumps后的数据字符串返回给客户端
        @last_editors: ChenXiaolei
        """
        self.reponse_common(1, desc, data)

    def reponse_json_error(self, desc='error'):
        """
        @description: 通用错误返回json结构
        @param desc: 返错误描述
        @return: 将dumps后的数据字符串返回给客户端
        @last_editors: ChenXiaolei
        """
        self.reponse_common(0, desc)

    def reponse_json_error_params(self, desc='params error'):
        """
        @description: 通用参数错误返回json结构
        @param desc: 返错误描述
        @return: 将dumps后的数据字符串返回给客户端
        @last_editors: ChenXiaolei
        """
        self.reponse_common(0, desc)

    def redirect_url(self,
                     url: str,
                     permanent: bool = False,
                     status: int = None):
        """
        @description: 用于异步handler直接进行页面重定向
        @param url: 需跳转到的url
        @param permanent: 表示该重定向为临时性的；如果为True，则该重定向为永久性。
        @param status: 默认302,当status被指定了值的话，那个该值将会作为HTTP返回给客户端的状态码；如果没有指定特定的值，那么根据上方的permanent状态，如果permanent为True，则该status返回301；如果permanent为False，则该status返回302。
        @return: 重定向
        @last_editors: ChenXiaolei
        """
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        self.redirect(url, permanent, status)


def filter_check_params(must_params, optional_params=None):
    """
    @description: 参数过滤装饰器 仅限handler使用,
                  提供参数的检查及获取参数功能
                  装饰器使用方法:
                  @param_filter("param_a","param_b","param_c","param_d")  或
                  @param_filter(["param_a","param_b"],["param_c","param_d"])
                  参数获取方法:
                  self.request_parms[param_key]
    @param must_params: 必须传递的参数集合
    @param optional_params: 可选传递的参数集合
    @last_editors: ChenXiaolei
    """
    def check_params(handler):
        def wrapper(self, **args):
            if not must_params and not optional_params:
                self.reponse_json_error_params()
                return
            self.request_parms = {}
            if type(must_params) == str:
                must_array = must_params.split(",")
            if optional_params and type(optional_params) == str:
                optional_array = optional_params.split(",")

            for param in must_array:
                value = self.get_param(param)
                if not value or value == "":
                    self.reponse_json_error_params()
                    return
                self.request_parms[param] = value

            if not optional_array and len(optional_array) > 0:
                for param in optional_array:
                    value = self.get_param(param)
                    if value and value != "":
                        self.request_parms[param] = value
            return handler(self, **args)

        return wrapper

    return check_params